package exercise_cellular_account;

public class CellularAccountVer7Test {

	public static void main(String[] args) {
		testAccountCreation();
		testMakeCall_Success();
		testMakeCall_Success_WithExtraMinutes();
		testMakeCall_Failure();
		testIsNearMax_Positive();
		testIsNearMax_Positive_LimitedPlan();
		testIsNearMax_Negative();
		testIsNearMax_Negative_LimitedPlan();
		testGetAmountDue();
		testToString();
	}

	private static void testAccountCreation() {
		System.out.println("-->testAccountCreation()");
		CellularAccountVer7 a1 = new CellularAccountVer7(500.0, 0.05);
		System.out.println("Expected: Amount Due=$0.00, Used=0.0 min., Max=500.0 min., Cost Per Min=0.05");
		System.out.println("  Actual: " + a1);
	}
	
	private static void testMakeCall_Success() {
		System.out.println("\n-->testMakeCall_Success()");
		CellularAccountVer7 a1 = new CellularAccountVer7(500.0, 0.05);
		System.out.println("Expected before makeCall(): Amount Due=$0.00, Used=0.0 min., Max=500.0 min., Cost Per Min=0.05");
		System.out.println("  Actual before makeCall(): " + a1);
		double callLength = 150.60;
		a1.makeCall(callLength);
		System.out.println("Expected after makeCall("+ callLength +"): Amount Due=$7.53, Used=150.6 min., Max=500.0 min., Cost Per Min=0.05");
		System.out.println("  Actual after makeCall("+ callLength +"): " + a1);
	}
	
	private static void testMakeCall_Success_WithExtraMinutes() {
		System.out.println("\n-->testMakeCall_Success_WithExtraMinutes()");
		CellularAccountVer7 a1 = new CellularAccountVer7(500.0, 0.05);
		a1.makeCall(700.0);
		System.out.println("Expected: Amount Due=$39.00, Used=600.0 min., Max=500.0 min., Cost Per Min=0.05");
		System.out.println("  Actual: " + a1);
	}
	
	private static void testMakeCall_Failure() {
		System.out.println("\n-->testMakeCall_Failure()");
		CellularAccountVer7 a1 = new CellularAccountVer7(500.0, 0.05);
		System.out.println("Expected before makeCall(): Amount Due=$0.00, Used=0.0 min., Max=500.0 min., Cost Per Min=0.05");
		System.out.println("  Actual before makeCall(): " + a1);
		double callLength = -150.60;
		a1.makeCall(callLength);
		System.out.println("Expected after makeCall("+ callLength +"): Amount Due=$0.00, Used=0.0 min., Max=500.0 min., Cost Per Min=0.05");
		System.out.println("  Actual after makeCall("+ callLength +"): " + a1);
	}
	
	private static void testIsNearMax_Positive() {
		System.out.println("\n-->testIsNearMax_Positive()");
		CellularAccountVer7 a1 = new CellularAccountVer7(500.0, 0.05);
		System.out.println("Expected: isNearMax before calls = false");
		System.out.println("  Actual: isNearMax before calls = " + a1.isNearMax());
		double minutesUsed = a1.getMinutesUsed();
		System.out.println("Minutes until maxed out = " + (500 - minutesUsed));
		a1.makeCall(460.0);
		System.out.println("Expected: isNearMax after calls = true");
		System.out.println("  Actual: isNearMax after calls = " + a1.isNearMax());
		minutesUsed = a1.getMinutesUsed();
		System.out.println("Minutes until maxed out = " + (500 - minutesUsed));
	}
	
	private static void testIsNearMax_Positive_LimitedPlan() {
		System.out.println("\n-->testIsNearMax_Positive_LimitedPlan()");
		CellularAccountVer7 a1 = new CellularAccountVer7(40.0, 0.05);
		System.out.println("Expected: isNearMax without calls = true");
		System.out.println("  Actual: isNearMax without calls = " + a1.isNearMax());
		double minutesUsed = a1.getMinutesUsed();
		System.out.println("Minutes until maxed out = " + (40.0 - minutesUsed));
	}
	
	private static void testIsNearMax_Negative() {
		System.out.println("\n-->testIsNearMax_Negative()");
		CellularAccountVer7 a1 = new CellularAccountVer7(500.0, 0.05);
		System.out.println("Expected: isNearMax without calls = false");
		System.out.println("  Actual: isNearMax without calls = " + a1.isNearMax());
		double minutesUsed = a1.getMinutesUsed();
		System.out.println("Minutes until maxed out = " + (500 - minutesUsed));
	}
	
	private static void testIsNearMax_Negative_LimitedPlan() {
		System.out.println("\n-->testIsNearMax_Negative_LimitedPlan()");
		CellularAccountVer7 a1 = new CellularAccountVer7(51.0, 0.05);
		System.out.println("Expected: isNearMax without calls = false");
		System.out.println("  Actual: isNearMax without calls = " + a1.isNearMax());
		double minutesUsed = a1.getMinutesUsed();
		System.out.println("Minutes until maxed out = " + (51.0 - minutesUsed));
	}
	
	private static void testGetAmountDue() {
		System.out.println("\n-->testGetAmountDue()");
		CellularAccountVer7 a1 = new CellularAccountVer7(500.0, 0.05);
		double accumulator = 0.00;
		for(int call = 0; call < 3; call++) {
			double callLength = (double) Math.random() * 100.0;
			a1.makeCall(callLength);
			accumulator += (callLength * 0.05);
			System.out.println(String.format("Expected Amount Due after call #%d: $%.2f", (call+1), (accumulator)));
			System.out.println(String.format("  Actual Amount Due after call #%d: $%.2f", (call+1), a1.getAmountDue()));
		}
	}
	
	private static void testToString() {
		System.out.println("\n-->testToString()");
		CellularAccountVer7 a1 = new CellularAccountVer7(500.0, 0.05);
		a1.makeCall(100.0);
		a1.makeCall(120.0);
		a1.makeCall(160.0);
		System.out.println("Expected: Amount Due=$19.00, Used=380.0 min., Max=500.0 min., Cost Per Min=0.05");
		System.out.println("  Actual: " + a1.toString());
	}
}
