package exercise_savings_account;

public class SavingsAccount3Test {

	/*
	 * Does not contain test methods for applyInterest, getServiceFeeTotal, isOverdrawn
	 * 
	 * Added tests for applyInterest and getServiceFeeTotal. isOverdrawn does not need tests because 
	 * it is not part of the class.
	 */
	public static void main(String[] args) {
		testAccountCreation_NoArgs();
		testAccountCreation_OnlyBalance();
		testAccountCreation_OnlyOwner();
		testAccountCreation_TwoArgs();
		testToString();
		testDeposit();
		testMultipleDeposits();		
		testDepositWithNegativeAmount();
		testWithdraw();
		testMultipleDepositsAndWithdrawals();		
		testWithdrawWithNegativeAmount();
		testApplyInterest_PositiveBalance();
		testApplyInterest_NegativeBalance();
		testServiceFee_PositiveBalance();
		testServiceFee_NegativeBalance();
		testMerge_PositiveBalance_SameOwner();
		testMerge_PositiveBalance_DifferentOwner();
		testMerge_NegativeBalance_SameOwner();
		testMerge_NegativeBalance_DifferentOwner();
	}
	
	private static void testAccountCreation_NoArgs() {
		System.out.println("-->testAccountCreation_NoArgs");
		SavingsAccount3 a1 = new SavingsAccount3();
		System.out.println("Expected: Owner= Anonymous, Balance=$0.00");
		System.out.println(String.format("  Actual: Owner= %s, Balance=$%,.2f", a1.getOwner(), a1.getBalance()));
	}
	
	private static void testAccountCreation_OnlyBalance() {
		System.out.println("\n-->testAccountCreation_OnlyBalance()");
		SavingsAccount3 a1 = new SavingsAccount3(1000.0);
		System.out.println("Expected: Owner= Anonymous, Balance=$1,000.00");
		System.out.println(String.format("  Actual: Owner= %s, Balance=$%,.2f", a1.getOwner(), a1.getBalance()));
	}
	
	private static void testAccountCreation_OnlyOwner() {
		System.out.println("\n-->testAccountCreation_OnlyOwner()");
		SavingsAccount3 a1 = new SavingsAccount3("Xeno");
		System.out.println("Expected: Owner= Xeno, Balance=$0.00");
		System.out.println(String.format("  Actual: Owner= %s, Balance=$%,.2f", a1.getOwner(), a1.getBalance()));
	}
	
	private static void testAccountCreation_TwoArgs() {
		System.out.println("\n-->testAccountCreation_TwoArgs()");
		SavingsAccount3 a1 = new SavingsAccount3(1000.0, "Xeno");
		System.out.println("Expected: Owner= Xeno, Balance=$1,000.00");
		System.out.println(String.format("  Actual: Owner= %s, Balance=$%,.2f", a1.getOwner(), a1.getBalance()));
	}

	private static void testToString() {
		System.out.println("\n-->testToString()");
		SavingsAccount3 a1 = new SavingsAccount3(1000.0, "Xeno");
		System.out.println("Expected: Account: balance=$1,000.00, serv fees=$0.00");
		System.out.println("  Actual: " + a1);
	}

	private static void testDeposit() {
		System.out.println("\n-->testDeposit()");
		SavingsAccount3 a1 = new SavingsAccount3(1000.0, "Xeno");
		a1.deposit(500.0);
		System.out.println("balance should be $1500, balance=$" + a1.getBalance());
	}

	private static void testDepositWithNegativeAmount() {
		System.out.println("\n-->testDepositWithNegativeAmount()");
		SavingsAccount3 a1 = new SavingsAccount3(1000.0, "Xeno");
		a1.deposit(-500.0);
		System.out.println("balance should be $1000, balance=$" + a1.getBalance());
	}

	private static void testMultipleDeposits() {
		System.out.println("\n-->testMultipleDeposits()");
		SavingsAccount3 a1 = new SavingsAccount3(1000.0, "Xeno");
		a1.deposit(500.0);
		a1.deposit(300.0);
		System.out.println("balance should be $1800, balance=$" + a1.getBalance());
	}

	private static void testWithdraw() {
		System.out.println("\n-->testWithdraw()");
		SavingsAccount3 a1 = new SavingsAccount3(1000.0, "Xeno");
		a1.withdraw(600.0);
		System.out.println("balance should be $400, balance=$" + a1.getBalance());
	}

	private static void testMultipleDepositsAndWithdrawals() {
		System.out.println("\n-->testMultipleDepositsAndWithdrawals()");
		SavingsAccount3 a1 = new SavingsAccount3(1000.0, "Xeno");
		a1.deposit(500.0);
		a1.withdraw(200.0);
		a1.deposit(400.0);
		a1.withdraw(1000.0);
		System.out.println("balance should be $700, balance=$" + a1.getBalance());
	}

	private static void testWithdrawWithNegativeAmount() {
		System.out.println("\n-->testWithdrawWithNegativeAmount()");
		SavingsAccount3 a1 = new SavingsAccount3(1000.0, "Xeno");
		a1.withdraw(-500.0);
		System.out.println("balance should be $1000, balance=$" + a1.getBalance());
	}

	private static void testApplyInterest_PositiveBalance() {
		System.out.println("\n-->testApplyInterest_PositiveBalance()");
		SavingsAccount3 a1 = new SavingsAccount3(1000.0, "Mark");
		System.out.println("Expected: Balance before interest is applied = $1000.00");
		System.out.println(String.format("  Actual: Balance before interest is applied = $%.2f", a1.getBalance()));
		a1.withdraw(200.0);
		a1.deposit(700.0);
		a1.applyInterest(0.05);
		System.out.println("Expected: Balance after interest was applied = $1575.00");
		System.out.println(String.format("  Actual: Balance after interest was applied = $%.2f", a1.getBalance()));
		
	}
	
	private static void testApplyInterest_NegativeBalance() {
		System.out.println("\n-->testApplyInterest_NegativeBalance()");
		SavingsAccount3 a1 = new SavingsAccount3(200.0, "Mark");
		a1.withdraw(250.0);
		System.out.println("Expected: Balance before interest is applied = $-70.00");
		System.out.println(String.format("  Actual: Balance before interest is applied = $%.2f", a1.getBalance()));
		a1.applyInterest(0.05);
		System.out.println("Expected: Balance after interest was applied = $-70.00");
		System.out.println(String.format("  Actual: Balance after interest was applied = $%.2f", a1.getBalance()));
	}
	
	private static void testServiceFee_PositiveBalance() {
		System.out.println("\n-->testServiceFee_PositiveBalance()");
		SavingsAccount3 a1 = new SavingsAccount3(1000.0, "Mark");
		a1.withdraw(250.0);
		a1.withdraw(250.0);
		System.out.println("Expected: Service Fee for the month is = $0.00");
		System.out.println(String.format("  Actual: Service Fee for the month is = $%.2f", a1.getServiceFeeTotal()));
	}
	
	private static void testServiceFee_NegativeBalance() {
		System.out.println("\n-->testServiceFee_NegativeBalance()");
		SavingsAccount3 a1 = new SavingsAccount3(500.0, "Mark");
		a1.withdraw(550.0);
		a1.deposit(100.0);
		a1.withdraw(200.0);
		System.out.println("Expected: Service Fee for the month is = $40.00");
		System.out.println(String.format("  Actual: Service Fee for the month is = $%.2f", a1.getServiceFeeTotal()));
	}
	
	private static void testMerge_PositiveBalance_SameOwner() {
		System.out.println("\n-->testMerge_PositiveBalance_SameOwner()");
		SavingsAccount3 a1 = new SavingsAccount3(500.0, "Mark");
		SavingsAccount3 a2 = new SavingsAccount3(100.0, "Mark");
		System.out.println("Expected: a1 before merge = Account: balance=$500.00, serv fees=$0.00");
		System.out.println("  Actual: a1 before merge = " + a1);
		a1.mergeAccount(a2);
		System.out.println("\nExpected: a1 after merge = Account: balance=$600.00, serv fees=$0.00");
		System.out.println("  Actual: a1 after merge = " + a1);
		
	}
	
	private static void testMerge_PositiveBalance_DifferentOwner() {
		System.out.println("\n-->testMerge_PositiveBalance_DifferentOwner()");
		SavingsAccount3 a1 = new SavingsAccount3(500.0, "Mark");
		SavingsAccount3 a2 = new SavingsAccount3(100.0, "Xeno");
		System.out.println("Expected: a1 before merge = Account: balance=$500.00, serv fees=$0.00");
		System.out.println("  Actual: a1 before merge = " + a1);
		a1.mergeAccount(a2);
		System.out.println("\nExpected: a1 after merge = Account: balance=$500.00, serv fees=$0.00");
		System.out.println("  Actual: a1 after merge = " + a1);
	}
	
	private static void testMerge_NegativeBalance_SameOwner() {
		System.out.println("\n-->testMerge_NegativeBalance_SameOwner()");
		SavingsAccount3 a1 = new SavingsAccount3(100.0, "Mark");
		a1.withdraw(120.0);
		SavingsAccount3 a2 = new SavingsAccount3(100.0, "Mark");
		System.out.println("Expected: a1 before merge = Account: balance=$-40.00, serv fees=$20.00");
		System.out.println("  Actual: a1 before merge = " + a1);
		a1.mergeAccount(a2);
		System.out.println("\nExpected: a1 after merge = Account: balance=$60.00, serv fees=$20.00");
		System.out.println("  Actual: a1 after merge = " + a1);
	}
	
	private static void testMerge_NegativeBalance_DifferentOwner() {
		System.out.println("\n-->testMerge_NegativeBalance_DifferentOwner()");
		SavingsAccount3 a1 = new SavingsAccount3(100.0, "Mark");
		a1.withdraw(120.0);
		SavingsAccount3 a2 = new SavingsAccount3(100.0, "Xeno");
		System.out.println("Expected: a1 before merge = Account: balance=$-40.00, serv fees=$20.00");
		System.out.println("  Actual: a1 before merge = " + a1);
		a1.mergeAccount(a2);
		System.out.println("\nExpected: a1 after merge = Account: balance=$-40.00, serv fees=$20.00");
		System.out.println("  Actual: a1 after merge = " + a1);
	}
}
